# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License.
"""Application Insights Telemetry Processor for Bots."""

import traceback
from typing import Dict
from applicationinsights import TelemetryClient
from botbuilder.core.bot_telemetry_client import BotTelemetryClient, TelemetryDataPointType
from .integration_post_data import IntegrationPostData

# pylint: disable=line-too-long
def bot_telemetry_processor(data, context) -> bool:
    """ Application Insights Telemetry Processor for Bot
    Traditional Web user and session ID's don't apply for Bots.  This processor
    replaces the identifiers to be consistent with Bot Framework's notion of
    user and session id's.

    Each event that gets logged (with this processor added) will contain additional
    properties.

    The following properties are replaced:
     - context.user.id    - The user ID that Application Insights uses to identify
                            a unique user.
     - context.session.id - The session ID that APplication Insights uses to
                            identify a unique session.

     In addition, the additional data properties are added:
     - activityId - The Bot Framework's Activity ID which represents a unique
                    message identifier.
     - channelId - The Bot Framework "Channel" (ie, slack/facebook/directline/etc)
     - activityType - The Bot Framework message classification (ie, message)

    :param data: Data from Application Insights
    :type data: telemetry item
    :param context: Context from Application Insights
    :type context: context object
    :returns:  bool -- determines if the event is passed to the server (False = Filtered).
    """
    post_data = IntegrationPostData().activity_json
    if post_data is None:
        # If there is no body (not a BOT request or not configured correctly).
        # We *could* filter here, but we're allowing event to go through.
        return True

    # Override session and user id
    from_prop = post_data['from'] if 'from' in post_data else None
    user_id = from_prop['id'] if from_prop is not None else None
    channel_id = post_data['channelId'] if 'channelId' in post_data else None
    conversation = post_data['conversation'] if 'conversation' in post_data else None
    conversation_id = conversation['id'] if 'id' in conversation else None
    context.user.id = channel_id + user_id
    context.session.id = conversation_id

    # Additional bot-specific properties
    if 'id' in post_data:
        data.properties["activityId"] = post_data['id']
    if 'channelId' in post_data:
        data.properties["channelId"] = post_data['channelId']
    if 'type' in post_data:
        data.properties["activityType"] = post_data['type']
    return True


class ApplicationInsightsTelemetryClient(BotTelemetryClient):
    """Application Insights Telemetry Client."""
    def __init__(self, instrumentation_key: str, telemetry_client: TelemetryClient = None):
        self._instrumentation_key = instrumentation_key
        self._client = telemetry_client if telemetry_client is not None \
            else TelemetryClient(self._instrumentation_key)
        # Telemetry Processor
        self._client.add_telemetry_processor(bot_telemetry_processor)


    def track_pageview(self, name: str, url: str, duration: int = 0, properties: Dict[str, object] = None,
                       measurements: Dict[str, object] = None) -> None:
        """
        Send information about the page viewed in the application (a web page for instance).
        :param name: the name of the page that was viewed.
        :param url: the URL of the page that was viewed.
        :param duration: the duration of the page view in milliseconds. (defaults to: 0)
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param measurements: the set of custom measurements the client wants to attach to this data item. (defaults to: None)
        """
        self._client.track_pageview(name, url, duration, properties, measurements)

    def track_exception(self, exception_type: type = None, value: Exception = None, tb: traceback = None,
                        properties: Dict[str, object] = None, measurements: Dict[str, object] = None) -> None:
        """
        Send information about a single exception that occurred in the application.
        :param exception_type: the type of the exception that was thrown.
        :param value: the exception that the client wants to send.
        :param tb: the traceback information as returned by :func:`sys.exc_info`.
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param measurements: the set of custom measurements the client wants to attach to this data item. (defaults to: None)
        """
        self._client.track_exception(exception_type, value, tb, properties, measurements)

    def track_event(self, name: str, properties: Dict[str, object] = None,
                    measurements: Dict[str, object] = None) -> None:
        """
        Send information about a single event that has occurred in the context of the application.
        :param name: the data to associate to this event.
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param measurements: the set of custom measurements the client wants to attach to this data item. (defaults to: None)
        """
        self._client.track_event(name, properties=properties, measurements=measurements)

    def track_metric(self, name: str, value: float, tel_type: TelemetryDataPointType = None,
                     count: int = None, min_val: float = None, max_val: float = None, std_dev: float = None,
                     properties: Dict[str, object] = None) -> NotImplemented:
        """
        Send information about a single metric data point that was captured for the application.
        :param name: The name of the metric that was captured.
        :param value: The value of the metric that was captured.
        :param tel_type: The type of the metric. (defaults to: TelemetryDataPointType.aggregation`)
        :param count: the number of metrics that were aggregated into this data point. (defaults to: None)
        :param min_val: the minimum of all metrics collected that were aggregated into this data point. (defaults to: None)
        :param max_val: the maximum of all metrics collected that were aggregated into this data point. (defaults to: None)
        :param std_dev: the standard deviation of all metrics collected that were aggregated into this data point. (defaults to: None)
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        """
        self._client.track_metric(name, value, tel_type, count, min_val, max_val, std_dev, properties)

    def track_trace(self, name: str, properties: Dict[str, object] = None, severity=None):
        """
        Sends a single trace statement.
        :param name: the trace statement.
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param severity: the severity level of this trace, one of DEBUG, INFO, WARNING, ERROR, CRITICAL
        """
        self._client.track_trace(name, properties, severity)

    def track_request(self, name: str, url: str, success: bool, start_time: str = None,
                      duration: int = None, response_code: str = None, http_method: str = None,
                      properties: Dict[str, object] = None, measurements: Dict[str, object] = None,
                      request_id: str = None):
        """
        Sends a single request that was captured for the application.
        :param name: The name for this request. All requests with the same name will be grouped together.
        :param url: The actual URL for this request (to show in individual request instances).
        :param success: True if the request ended in success, False otherwise.
        :param start_time: the start time of the request. The value should look the same as the one returned by :func:`datetime.isoformat()` (defaults to: None)
        :param duration: the number of milliseconds that this request lasted. (defaults to: None)
        :param response_code: the response code that this request returned. (defaults to: None)
        :param http_method: the HTTP method that triggered this request. (defaults to: None)
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param measurements: the set of custom measurements the client wants to attach to this data item. (defaults to: None)
        :param request_id: the id for this request. If None, a new uuid will be generated. (defaults to: None)
        """
        self._client.track_request(name, url, success, start_time, duration, response_code, http_method, properties,
                                   measurements, request_id)

    def track_dependency(self, name: str, data: str, type_name: str = None, target: str = None, duration: int = None,
                         success: bool = None, result_code: str = None, properties: Dict[str, object] = None,
                         measurements: Dict[str, object] = None, dependency_id: str = None):
        """
        Sends a single dependency telemetry that was captured for the application.
        :param name: the name of the command initiated with this dependency call. Low cardinality value. Examples are stored procedure name and URL path template.
        :param data: the command initiated by this dependency call. Examples are SQL statement and HTTP URL with all query parameters.
        :param type_name: the dependency type name. Low cardinality value for logical grouping of dependencies and interpretation of other fields like commandName and resultCode. Examples are SQL, Azure table, and HTTP. (default to: None)
        :param target: the target site of a dependency call. Examples are server name, host address. (default to: None)
        :param duration: the number of milliseconds that this dependency call lasted. (defaults to: None)
        :param success: true if the dependency call ended in success, false otherwise. (defaults to: None)
        :param result_code: the result code of a dependency call. Examples are SQL error code and HTTP status code. (defaults to: None)
        :param properties: the set of custom properties the client wants attached to this data item. (defaults to: None)
        :param measurements: the set of custom measurements the client wants to attach to this data item. (defaults to: None)
        :param id: the id for this dependency call. If None, a new uuid will be generated. (defaults to: None)
        """
        self._client.track_dependency(name, data, type_name, target, duration, success, result_code, properties,
                                      measurements, dependency_id)

    def flush(self):
        """Flushes data in the queue. Data in the queue will be sent either immediately irrespective of what sender is
        being used.
        """
        self._client.flush()
